package me.chriskyle.fluttertoast;

import android.content.Context;
import android.graphics.PorterDuff;
import android.graphics.drawable.Drawable;
import android.os.Build;
import android.os.Handler;
import android.view.Gravity;
import android.view.MotionEvent;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

public final class FluttertoastPlugin implements MethodCallHandler {

    Context context;
    Toast toast;

    FluttertoastPlugin(Context context) {
        this.context = context;
    }

    public static void registerWith(Registrar registrar) {
        final MethodChannel channel = new MethodChannel(registrar.messenger(), "chriskyle/fluttertoast");
        channel.setMethodCallHandler(new FluttertoastPlugin(registrar.context()));
    }

    @Override
    public void onMethodCall(MethodCall call, final Result result) {
        if (call.method.equals("showToast")) {
            String msg = call.argument("msg").toString();
            String duration = call.argument("duration").toString();
            String gravity = call.argument("gravity").toString();
            Number textcolor = call.argument("textColor");
            Number textSize = call.argument("fontSize");
            Number backgroundColor = call.argument("backgroundColor");

            toast = Toast.makeText(context, msg, Toast.LENGTH_SHORT);
            toast.setText(msg);

            if (duration.equals("long")) {
                toast.setDuration(Toast.LENGTH_LONG);
            } else {
                toast.setDuration(Toast.LENGTH_SHORT);
            }

            switch (gravity) {
            case "bottom":
                toast.setGravity(Gravity.BOTTOM, 0, 150);
                break;
            default:
                toast.setGravity(Gravity.CENTER, 0, 0);
            }

            final TextView text = toast.getView().findViewById(android.R.id.message);
            if (textSize != null)
                text.setTextSize(textSize.floatValue());

            if (backgroundColor != null) {
                Drawable shapeDrawable;
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                    shapeDrawable = context.getResources().getDrawable(R.drawable.toast_bg, context.getTheme());
                } else {
                    shapeDrawable = context.getResources().getDrawable(R.drawable.toast_bg);
                }

                if (shapeDrawable != null) {
                    shapeDrawable.setColorFilter(backgroundColor.intValue(), PorterDuff.Mode.SRC_IN);
                    if (Build.VERSION.SDK_INT <= 27) {
                        toast.getView().setBackground(shapeDrawable);
                    } else {
                        toast.getView().setBackground(null);
                        text.setBackground(shapeDrawable);
                    }
                }
            }

            if (textcolor != null) {
                text.setTextColor(textcolor.intValue());
            }

            toast.show();
            result.success(true);
        } else if (call.method.equals("cancel")) {
            if (toast != null)
                toast.cancel();
            result.success(true);
        } else {
            result.notImplemented();
        }
    }
}
