import 'dart:async';
import 'dart:ui';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:meta/meta.dart';
import 'package:flutter/foundation.dart';

enum ToastDuration { LENGTH_SHORT, LENGTH_LONG }

enum ToastGravity { BOTTOM, CENTER }

class Fluttertoast {
  static const MethodChannel _channel =
      const MethodChannel('chriskyle/fluttertoast');

  static Fluttertoast _instance;

  static Fluttertoast get instance {
    if (_instance == null) {
      _instance = Fluttertoast._create();
    }
    return _instance;
  }

  Fluttertoast._create();

  static Future<bool> showToast({
    @required String msg,
    ToastDuration toastDuration,
    ToastGravity toastGravity,
    int durationTime = 2,
    double fontSize = 16.0,
    Color backgroundColor,
    Color textColor,
  }) async {
    String duration = "short";
    if (toastDuration == ToastDuration.LENGTH_LONG) {
      duration = "long";
    }

    String gravity = "bottom";
    if (toastGravity == ToastGravity.CENTER) {
      gravity = "center";
    } else {
      gravity = "bottom";
    }

    if (textColor == null) {
      textColor = Colors.white;
    }

    if (backgroundColor == null &&
        defaultTargetPlatform == TargetPlatform.iOS) {
      backgroundColor = Colors.black;
    }

    final Map<String, dynamic> params = <String, dynamic>{
      'msg': msg,
      'duration': duration,
      'gravity': gravity,
      'durationTime': durationTime,
      'backgroundColor': backgroundColor != null ? backgroundColor.value : null,
      'textColor': textColor != null ? textColor.value : null,
      'fontSize': fontSize,
    };

    bool isSuccess = await _channel.invokeMethod('showToast', params);
    return isSuccess;
  }

  static Future<bool> cancel() async {
    bool isSuccess = await _channel.invokeMethod("cancel");
    return isSuccess;
  }
}
